# -*- coding: utf-8 -*-
import logging
import sys
from conf.config import LOG_LEVEL


logger = logging.getLogger("dingding_notify")

formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
# formatter = logging.Formatter('[%(asctime)s] %(name)s %(levelname)s %(pathname)s %(lineno)d %(message)s')

stream_handler = logging.StreamHandler(sys.stderr)
stream_handler.setFormatter(formatter)

logger.addHandler(stream_handler)

logger.setLevel(LOG_LEVEL)


# -*- coding: utf-8 -*-
import time
import random
import threading
from gevent.queue import JoinableQueue


from inner_lib.func_ext import get_md5, http_request, message_format, singleton
from conf.config import MAXSIZE


@singleton
class NoticeClient(object):
    init_flag = False

    def __init__(self, app_key, app_secret, api_url):
        if not self.init_flag:  # 防止重复执行init方法
            self.api_url = api_url
            self.app_key = app_key
            self.app_secret = app_secret
            self.req_q = JoinableQueue(MAXSIZE)
            self.init_flag = True
            t1 = threading.Thread(target=http_request, args=[self.api_url, self.req_q])
            t1.start()
        else:
            return

    def sys_params(self, body):
        """构造请求参数参数"""
        time.sleep(1)
        now = int(time.time())
        auth_key = '%d-%s-%s' % (now, self.app_secret, self.app_key)
        auth_key_md5 = get_md5(auth_key)
        auth_str = auth_key_md5[0:4] + str(random.randint(100, 999)) + auth_key_md5[4:24] + str(
            random.randint(10000, 99999)) + auth_key_md5[24:]
        _params = {
            "key": self.app_key,
            "auth_str": auth_str,
            "timestamp": now,
            "req_msg": body,
        }
        return _params

    def send(self, data, to_users):
        to_users = "|".join(to_users)
        data = message_format(data)
        body = {
            "to_user": to_users,
            "content": data
        }
        _params = self.sys_params(body)
        self.req_q.put(_params)

        return True


if __name__ == '__main__':
    pass


# -*- coding: utf-8 -*-
import time
import unittest

from dingding_client.notice_client import NoticeClient


class SendMessageToDingDingTestCase(unittest.TestCase):
    def setUp(self):
        self.app_key = ""
        self.app_secret = ""
        self.api_url = ""

    def testSend(self):
        client = NoticeClient(app_key=self.app_key, app_secret=self.app_secret, api_url=self.api_url)
        try:
            # 第一个参数是一个字典，表示要发送的数据
            # 第二个参数是一个列表， 表示要发送给谁
            client.send({"message": "测试数据"}, ["s4261"])
            client.send({"message": "测试数据"}, ["s4261"])
            print "测试"
        except Exception as e:
            print(e)

        while 1:
            time.sleep(10)


# -*- coding: UTF-8 -*-
from __future__ import with_statement
import time
import urllib2
import hashlib
import socket
socket.setdefaulttimeout(15)
import threading
import json

from conf.config import RETRY_COUNT, TIME_STEP
from inner_lib.log import *


def get_md5(string):
    try:
        m = hashlib.md5()
        m.update(string)
        dest = m.hexdigest()
        return dest
    except:
        return False


def http_request(url, req_q, headers={'user-agent': 'shopex/spider'}):
    cookies = urllib2.HTTPCookieProcessor()
    opener = urllib2.build_opener(cookies)
    retry_count = RETRY_COUNT
    '''
        数据请求处理
    '''
    while 1:
        try:
            data = json.dumps(req_q.get(block=False))
        except Exception, e:
            time.sleep(2)
            continue
        while retry_count >= 0:
            try:
                request = urllib2.Request(url=url, headers=headers, data=data)
                response = opener.open(request).read()
                response = json.loads(response)
                status = response.get("errmsg")
            except Exception, e:
                response = e
                status = 'fail'
                logger.error(e)

            if status != "ok":  # 尝试重发
                logger.error(response)
                retry_count -= 1
                continue
            else:
                time.sleep(TIME_STEP)
                break


def message_format(data):
    event_time = time.strftime("%Y-%m-%d %H:%M:%S", data.get("timestamp", time.localtime()))
    event_main = data.get("main", "")
    event_service = data.get("service", "")
    event_message = data.get("message", "")
    return "事件产生时间:%s  事件产生主体:%s  所属服务:%s  事件内容:%s" % (event_time, event_main, event_service, event_message)


def singleton(cls):
    _instance = {}
    _instance_lock = threading.Lock()

    def wrappers(*args, **kwargs):
        # 双重检查锁定创建单例
        if not _instance.get(cls.__name__):
            with _instance_lock:
                if not _instance.get(cls):
                    _instance[cls.__name__] = cls(*args, **kwargs)
                    return _instance[cls.__name__]
        else:
            return _instance.get(cls.__name__)

    return wrappers


# -*- coding: UTF-8 -*-

import sys
reload(sys)  # Python2.5 初始化后会删除 sys.setdefaultencoding 这个方法，我们需要重新载入
sys.setdefaultencoding('utf-8')

from setuptools import setup, find_packages

setup(
    name="shopex_dingding_note_2.x",
    version=1.3,
    description="ShopEx dingding notify for python",
    long_description_content_type=open("README.md").read(),
    packages=find_packages(),
    author="ShopEx",
    author_email="xuhongtao@shopex.cn",
    license="LGPL",
    url="https://github.com/magic119/shopex_dingding_note_2.x.git",
    platforms=["README.md"],
    python_requires=">=2.7.0, <2.7.18",
)